@@ -81,6 +81,8 @@ def main():
     parser = argparse.ArgumentParser(
         description='Install glean init components')
 
+    parser.add_argument("-n", "--use-nm", help="Use NetworkManager",
+                        action="store_true")
     parser.add_argument("-q", "--quiet", help="Be very quiet",
                         action="store_true")
 
@@ -95,25 +97,55 @@ def main():
     if os.path.exists('/etc/gentoo-release'):
         log.info('installing openrc services')
         install('glean.openrc', '/etc/init.d/glean')
-        subprocess.call(['rc-update', 'add', 'glean', 'boot'])
-    if os.path.exists('/usr/lib/systemd'):
+    # Needs to check for the presence of systemd and systemctl
+    # as apparently some packages may stage systemd init files
+    # when systemd is not present.
+    # We also cannot check the path for the init pid as the pid
+    # may be wrong as install is generally executed in a chroot
+    # with diskimage-builder.
+
+    if (os.path.exists('/usr/lib/systemd/system') and
+            (os.path.exists('/usr/bin/systemctl') or
+             os.path.exists('/bin/systemctl'))):
         p = _find_gleansh_path()
 
         log.info("Installing systemd services")
         log.info("glean.sh in %s" % p)
 
-        install(
-            'glean@.service',
-            '/usr/lib/systemd/system/glean@.service',
-            mode='0644',
-            replacements={'GLEANSH_PATH': p})
+        if os.path.exists('/etc/gentoo-release'):
+            install(
+                'glean-networkd.service',
+                '/lib/systemd/system/glean.service',
+                mode='0644',
+                replacements={'GLEANSH_PATH': p})
+            subprocess.call(['systemctl', 'enable', 'glean.service'])
+        else:
+            log.info("Installing %s NetworkManager support" %
+                     "with" if args.use_nm else "without")
+            if args.use_nm:
+                service_file = 'glean-nm@.service'
+            else:
+                service_file = 'glean@.service'
+            install(
+                service_file,
+                '/usr/lib/systemd/system/glean@.service',
+                mode='0644',
+                replacements={'GLEANSH_PATH': p})
         install(
             'glean-udev.rules',
             '/etc/udev/rules.d/99-glean.rules',
             mode='0644')
+        if args.use_nm:
+            # glean handles resolv.conf, so this turns off nm touching
+            # it
+            install('nm-no-resolv-handling.conf',
+                    '/etc/NetworkManager/conf.d/nm-no-resolv-handling.conf',
+                    mode='0644')
     elif os.path.exists('/etc/init'):
         log.info("Installing upstart services")
         install('glean.conf', '/etc/init/glean.conf')
+    elif os.path.exists('/sbin/rc-update'):
+        subprocess.call(['rc-update', 'add', 'glean', 'boot'])
     else:
         log.info("Installing sysv services")
         install('glean.init', '/etc/init.d/glean')
